/****************************************************************************
Title:    Access serial EEPROM 24C02 using I2C interace
Author:   Peter Fleury <pfleury@gmx.ch> http://jump.to/fleury
          Peter Recktenwald http://asuro.pytalhost.de ASURO Porting
Software: AVR-GCC 3.3
Hardware: ASURO Robo ATMEGA8 at 8MHz

Description:
    This example shows how the I2C library i2cmaster.S can be used to 
    access a 8-Bit Port Extender.
    Based on Atmel Application Note AVR300, adapted to AVR-GCC C interface

*****************************************************************************/
#include <avr/io.h>
#include "i2cmaster.h"
#include "asuro.h"


#define Dev8574   0x40      // device address of PCF 8574, see datasheet


int main(void)
{
  unsigned char ret;
  unsigned char val;
    
  Init();                                    // init Asuro
  i2c_init();                                // init I2C interface
  val = 0;

  SerWrite("I2C Demo", 8);
  SerWrite("\r\n", 2);

  while(1)
  {	    
    ret = i2c_start(Dev8574+I2C_WRITE);       // set device address and write mode
    if ( ret ) 
    {
      /* failed to issue start condition, possibly no device found */
      i2c_stop();
      StatusLED(RED);
      PrintInt(ret);
      SerWrite("\r\n", 2);
      Msleep(500);
    }
    else 
    {
      /* issuing start condition ok, device accessible */
      i2c_write(val);                       // write data
      i2c_stop();                           // set stop conditon = release bus
      StatusLED(GREEN);
      PrintInt(val);
      SerWrite("\r\n", 2);
      val++;
      Msleep(500);
    }
  }
  return 0;
}
